
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import java.util.Random;

public class MyCanvas extends Canvas implements Runnable {

    public static final int USER_EXIT = 1; // user aborted test
    public static final int EXIT_DONE = 2; // normal end of test

    // konstanty modelu
    public static final int LOOPS = 100;
    public static final int DRAWS = 8;

    public static final int NUMBER_COUNT = 32;
    public static final int DIVISOR_COUNT = 8;

    // animacni konstanty
    public static final int WAIT_TIME = 50;

    // barvy
    public static final int BACKGROUND = 0x00FFFFFF;
    public static final int FOREGROUND = 0x00000000;

    public long elapsed = 0;
    public int exitStatus;

    private Thread animationThread;
    private OptiMidlet midlet;
    private boolean finished;

    private long started;
    private long frameStarted;
    private long frameTime;

    private int loopCounter;

    private Random randomGenerator = new Random();

    // new variables
    public static int WIDTH;
    public static int HEIGHT;
    public static final Font MY_FONT = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_BOLD | Font.STYLE_ITALIC, Font.SIZE_SMALL);
    public static final int TEXT_ANCHOR = Graphics.TOP | Graphics.HCENTER;
    public static final int MESSAGE_ANCHOR = Graphics.TOP | Graphics.LEFT;
    public static final String MESSAGE = "ms per frame: ";
    private static final StringBuffer buffer = new StringBuffer();
    private static int messageLength;
    private static final int MAX_RANDOM = 250;
    private static int[] randomX;
    private static int[] randomY;
    private static int rnd;
    private long oldTime;
    private Image messageImage;
    private Graphics messageGraphics;

    private static byte MAX_STRING_WIDTH;
    private static byte MAX_STRING_HEIGHT;

    public MyCanvas(OptiMidlet midlet) {
        this.midlet = midlet;

        init();
    }

    private void init() {
        WIDTH = getWidth();
        HEIGHT = getHeight();

        buffer.append(MESSAGE).append(" 00000");
        messageLength = MESSAGE.length();

        MAX_STRING_WIDTH = (byte) MY_FONT.stringWidth(buffer.toString());
        MAX_STRING_HEIGHT = (byte) MY_FONT.getHeight();

        messageImage = Image.createImage(MAX_STRING_WIDTH, MAX_STRING_HEIGHT);
        messageGraphics = messageImage.getGraphics();
        messageGraphics.setFont(MY_FONT);

        randomX = new int[MAX_RANDOM];
        randomY = new int[MAX_RANDOM];
        rnd = 0;

        for (int i = MAX_RANDOM; --i >= 0;) {
            randomX[i] = getRandom(WIDTH);
            randomY[i] = getRandom(HEIGHT);
        }
    }

    public synchronized void start() {

        frameStarted = System.currentTimeMillis();
        started = frameStarted;

        loopCounter = 0;
        finished = false;
        exitStatus = EXIT_DONE;

        animationThread = new Thread(this);
        animationThread.start();
    }

    public void run() {

        Thread currentThread = Thread.currentThread();

        try {
            while (animationThread == currentThread && midlet.running && !finished) {

                frameTime = System.currentTimeMillis() - frameStarted;
                frameStarted = System.currentTimeMillis();

                // toto volani reprezentuje model aplikace
                work();

                repaint();

                synchronized (this) {
                    wait(WAIT_TIME);
                }

                finished = (++loopCounter > LOOPS);
            }
        } catch (InterruptedException ie) {
            OptiMidlet.debug("perueno");
        }

        // celkovy cas
        elapsed = System.currentTimeMillis() - started;

        // zpet na midlet
        midlet.exitCanvas(exitStatus);
    }

    // simulace paint
    public void paint(Graphics g) {

        // prepare string
        if (oldTime != frameTime) {
            buffer.delete(messageLength, buffer.length());
            buffer.append(frameTime).toString();
            messageGraphics.setColor(BACKGROUND);
            messageGraphics.fillRect(0, 0, MAX_STRING_WIDTH, MAX_STRING_HEIGHT);
            messageGraphics.setColor(FOREGROUND);
            messageGraphics.drawString(buffer.toString(), 0, 0, MESSAGE_ANCHOR);
        }

        g.setColor(BACKGROUND);
        g.fillRect(0, 0, WIDTH, HEIGHT);

        for (int i = DRAWS; --i >= 0;) {
            rnd = (rnd + 1) % MAX_RANDOM;
            g.drawImage(messageImage, randomX[rnd], randomY[rnd], TEXT_ANCHOR);
        }

        oldTime = frameTime;
    }

    public static final int work() {

        int mul;
        int someGlobalConstant1 = 0;

        mul = doModel(0);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (0 * i + mul);}
        mul = doModel(1);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (1 * i + mul);}
        mul = doModel(2);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (2 * i + mul);}
        mul = doModel(3);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (3 * i + mul);}
        mul = doModel(4);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (4 * i + mul);}
        mul = doModel(5);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (5 * i + mul);}
        mul = doModel(6);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (6 * i + mul);}
        mul = doModel(7);
        for (int i = 0; i < NUMBER_COUNT; i++) {someGlobalConstant1 += (7 * i + mul);}

        return someGlobalConstant1;
    }

    public static final int doModel(int n) {

        int someGlobalConstants2 = 0;

        for (int i = 0; i < n; i++) {
            someGlobalConstants2 <<= 1;
        }

        return someGlobalConstants2;
    }

    // prekryj keyReleased metodu
    public void keyReleased(int keyCode) {
        exitStatus = USER_EXIT;
        midlet.running = false;
    }

    private int getRandom(int bound) {
        return Math.abs(randomGenerator.nextInt() % bound);
    }
}
