import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import java.util.Random;

public class MyCanvas extends Canvas implements Runnable {

    public static final int USER_EXIT = 1; // user aborted test
    public static final int EXIT_DONE = 2; // normal end of test

    // konstanty modelu
    public static final int LOOPS = 100;
    public static final int DRAWS = 8;

    public static final int NUMBER_COUNT = 32;
    public static final int DIVISOR_COUNT = 8;

    // animacni konstanty
    public static final int WAIT_TIME = 50;

    // barvy
    public static final int BACKGROUND = 0x00FFFFFF;
    public static final int FOREGROUND = 0x00000000;

    public long elapsed = 0;
    public int exitStatus;

    private Thread animationThread;
    private OptiMidlet midlet;
    private boolean finished;

    private long started;
    private long frameStarted;
    private long frameTime;

    private int loopCounter;

    private Random randomGenerator = new Random();

    private int mul;
    private int someGlobalConstant1;
    private int someGlobalConstants2;

    public MyCanvas(OptiMidlet midlet) {
        this.midlet = midlet;
    }

    public synchronized void start() {

        frameStarted = System.currentTimeMillis();
        started = frameStarted;

        loopCounter = 0;
        finished = false;
        exitStatus = EXIT_DONE;

        animationThread = new Thread(this);
        animationThread.start();
    }

    public void run() {

        Thread currentThread = Thread.currentThread();

        try {
            while (animationThread == currentThread && midlet.running && !finished) {

                frameTime = System.currentTimeMillis() - frameStarted;
                frameStarted = System.currentTimeMillis();

                // toto volani reprezentuje model aplikace
                work();

                repaint();

                synchronized (this) {
                    wait(WAIT_TIME);
                }

                finished = (++loopCounter > LOOPS);
            }
        } catch (InterruptedException ie) {
            OptiMidlet.debug("perueno");
        }

        // celkovy cas
        elapsed = System.currentTimeMillis() - started;

        // zpet na midlet
        midlet.exitCanvas(exitStatus);
    }

    // simulace paint
    public void paint(Graphics g) {

        g.setColor(BACKGROUND);
        g.fillRect(0, 0, getWidth(), getHeight());
        g.setColor(FOREGROUND);
        g.setFont(Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_BOLD | Font.STYLE_ITALIC, Font.SIZE_SMALL));

        for (int i = 0; i < DRAWS; i++) {
            g.drawString(frameTime + " ms per frame", getRandom(getWidth()), getRandom(getHeight()), Graphics.TOP | Graphics.HCENTER);
        }
    }

    public synchronized int work() {

        someGlobalConstant1 = 0;

        for (int j = 0; j < DIVISOR_COUNT; j++) {
            for (int i = 0; i < NUMBER_COUNT; i++) {
                mul = doModel(j);
                someGlobalConstant1 += mathOperation(j, i, mul);
            }
        }

        return someGlobalConstant1;
    }

    public synchronized int doModel(int n) {

        for (int i = 0; i < n; i++) {
            someGlobalConstants2 *= 2;
        }

        return someGlobalConstants2;
    }

    public synchronized int mathOperation(int a, int b, int c) {
        return a * b + c;
    }

    // prekryj keyReleased metodu
    public void keyReleased(int keyCode) {
        exitStatus = USER_EXIT;
        midlet.running = false;
    }

    private int getRandom(int bound) {
        return Math.abs(randomGenerator.nextInt() % bound);
    }
}
